#! /usr/bin/env python3.4
# -*- coding:Utf8 -*-
# ***************************************************************************
# *      Pratical method for determing the minimum embedding dimension      *
# *                     if a scalar time series                             *
# *           L. Cao, Physica D, 110 (1 & 2), 43-52, 1997.                  *
# *                                                                         *
# *   Notices:   This program is ONLY used for your researches.             *
# *                                                                         *
# *   Liangyue Cao   March,1997.  Version Python & C Eric Foucault VI 2022  *
# ***************************************************************************
# ***************************************************************************
# *     ndp  is the number of data points used for your calculation;
# *     ndp  should be smaller than 100000; 
# *     tau  is the time-delay used for phase space reconstructions;
# *     maxd is the maximum dimension which you want to test;
# *     maxd should be smaller than 50.
#***************************************************************************
# Importation de fonctions externes :
import os, time
import matplotlib.pyplot as plt
import numpy as np
##################################################
def listfichiers(path): 
    fichier=[] 
    for root, dirs, files in os.walk(path): 
        for i in files: 
            fichier.append(os.path.join(root, i)) 
#    print(fichier[0:2])
        print(files)
    return fichier
############# C library EF #########################
import ctypes as ct
from numpy.ctypeslib import ndpointer
#
_lib = ct.cdll.LoadLibrary("./calcul_dim_ef1.so")    #   
_lib.calc_dim.argtypes = [ ct.c_int, ct.c_int, ct.c_int, np.ctypeslib.ndpointer(dtype = np.float64), ndpointer(ct.c_double), ndpointer(ct.c_double)]
_lib.calc_dim.restype = ct.c_void_p
#
########## call the C routine  #####
def calcul_dim(d, ng, tau, x):
    aaa=np.zeros(1, dtype=float)       # 
    yaaa=np.zeros(1, dtype=float)       # 
    _lib.calc_dim(d, ng, tau, x, aaa, yaaa)
    return aaa, yaaa
################################
# Reading the data file :
rep_cour = os.getcwd()
#print('Emplacement du code :',rep_cour)
emplacement=rep_cour #
data_file='measurement'   # data file
fic_brut=open(emplacement+'/'+data_file+'.dat','r')	# open data file
# ligne=[fic_brut.readline() for i in range(35)]     # 
#print('ligne=',fic_brut.readline())
fic=fic_brut.readlines()
fic_brut.close()				#ferme le fichier de data
nbpts=len(fic)
print('nbpts=',nbpts)
x=[0]
for pt in range(nbpts):
    ligne=fic[pt].split()
    x = np.append(x, [float(ligne[0]),float(ligne[1])])
# ################### plot data   #######################################
fig,ax0 = plt.subplots()   #  tracé delta(P)
plt.title(emplacement+'/'+data_file+'.dat', fontsize='x-small')
ax0.plot(x,label=r'data x')  # 
ax0.set_ylabel(r'x (t)')
ax0.set_xlabel(r'(t)')
# plt.xlim((0, tps[3*period]))   # set the xlim to bottom, top
plt.legend(loc='best', fontsize='x-small')
fig.tight_layout()
##################### save figure  ####################
fig.savefig(emplacement+'/'+data_file+'.pdf', format='pdf') 
####################################################################
plt.show()
####################################################################
a = np.zeros(50, dtype=np.float64)       # 
ya= np.zeros(50, dtype=np.float64)       # 
v = np.zeros(50, dtype=np.float64)       # 
Ndp = min(10000, nbpts)
#############
fichier_e1=open('e10.txt','w', encoding = "ISO-8859-1")	#ouvre le fichier 
fichier_e2=open('e20.txt','w', encoding = "ISO-8859-1")	#ouvre le fichier 
tau = 15
maxd = 10
debut = time.process_time()
### dimension 1
ng = Ndp - tau
# ng=10000-cte
dim = time.process_time()
################ d=1 ############################
aa=0
yaa=0
for i in np.arange(1, ng+1) :
    v0=1e+30
    for j in np.arange(1, ng+1) :
        if j != i :
            vij = np.abs(x[i] - x[j]) 
            if ((vij < v0) and (vij != 0)) :
                n0=j
                v0=vij
    ctey = np.abs(x[i+tau]-x[n0+tau])
    aa += max(v0, ctey)/v0
    yaa += ctey
a[1]=aa/ng
ya[1]=yaa/ng
# print('temps pour la dimension ', d,'={:8.2f} mn'.format((time.process_time() - dim)/60))
print('a[1]=', a[1],' ya[1]=', ya[1] )
###
################# d 2 -> maxd #############################
for d in np.arange(2, maxd+1) :
    a[d] = 0
    ya[d] = 0
    cte=d*tau
    ng = Ndp - cte
    # ng=10000 - cte
########### calling  C procedure ###############
    aa, yaa = calcul_dim(d, ng, tau, x)
###################################################    
    a[d]=aa/ng
    ya[d]=yaa/ng
    if d >= 2 :
        print(d-1,' ', a[d]/a[d-1], file=fichier_e1)
        print(d-1,' ', ya[d]/ya[d-1], file=fichier_e2)
        # print(d-1, ' ', a[d]/a[d-1], ' ',ya[d]/ya[d-1],' time spent for the calculation = {:8.2f} minute'.format((time.process_time() - dim)/60))
        print(d-1, ' ', a[d]/a[d-1], ' ',ya[d]/ya[d-1])
fichier_e1.close()
fichier_e2.close()
print('temps spent ={:8.2f} mn'.format((time.process_time() - debut)/60))
######################## plot dimension  ####################################
fig,ax1 = plt.subplots()   #  tracé delta(P)
plt.title(emplacement+'/'+data_file+'.dat', fontsize='x-small')
ax1.plot(a[2:maxd+1]/a[1:maxd],'-x',label=r'E1')  # 
ax1.plot(ya[2:maxd+1]/ya[1:maxd],'-+',label=r'E2')  # 
ax1.set_ylabel(r'E1 & E2')
ax1.set_xlabel(r'Dimension')
# plt.xlim((0, tps[3*period]))   # set the xlim to bottom, top
plt.legend(loc='best', fontsize='x-small')
fig.tight_layout()
##################### save figure  ###########################
fig.savefig(emplacement+'/'+data_file+'_E(d)).pdf', format='pdf') 
####################################################################
plt.show()
###

